package com.sleepycat.je.log;

import java.nio.ByteBuffer;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.cleaner.UtilizationTracker;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.log.LogManager.LogResult;

public privileged aspect ObserveLogAspect {
	private CheckpointMonitor checkpointMonitor;

	after(EnvironmentImpl envImpl) throws DatabaseException :
		execution(LogManager.new(EnvironmentImpl, boolean)) && args(envImpl,boolean){

		checkpointMonitor = new CheckpointMonitor(envImpl);
	}

	before(LogResult logResult):
		execution(long LogManager.hook_processLogResult(LogResult)) && args(logResult){

		if (logResult.wakeupCheckpointer) {
			checkpointMonitor.activate();
		}

	}

	boolean LogResult.wakeupCheckpointer;

	LogResult around(LogManager logManager, LoggableObject item,
			boolean marshallOutsideLatch, ByteBuffer marshalledBuffer):
		execution(LogResult LogManager.logInternal(LoggableObject, boolean,boolean, boolean, long,boolean, ByteBuffer,UtilizationTracker)) &&
		args(item,boolean,boolean, boolean, long,marshallOutsideLatch, marshalledBuffer,UtilizationTracker) &&
		this(logManager)
	{

		LogResult lr = proceed(logManager, item, marshallOutsideLatch,
				marshalledBuffer);

		int entrySize = logManager.getEntrySize(item, marshallOutsideLatch,
				marshalledBuffer);

		lr.wakeupCheckpointer = checkpointMonitor.recordLogWrite(entrySize,
				item);
		return lr;
	}

}
