package com.sleepycat.je.recovery;

import com.sleepycat.je.CheckpointConfig;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.config.EnvironmentParams;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.tree.IN;
import com.sleepycat.je.utilint.DaemonThread;

public class CheckpointerDaemon extends DaemonThread {

	private EnvironmentImpl envImpl;
	private Checkpointer checkpointer;

	public CheckpointerDaemon(Checkpointer checkpointer,String name) throws DatabaseException {
//CP
		super(hook_getInterval(checkpointer), name, checkpointer.getEnv());
		this.checkpointer=checkpointer;
		this.envImpl = checkpointer.getEnv();
	}

	private static long hook_getInterval(Checkpointer checkpointer2) throws DatabaseException {
		return 0;
	}

	/**
	 * Return the number of retries when a deadlock exception occurs.
	 */
	protected int nDeadlockRetries() throws DatabaseException {

		return envImpl.getConfigManager().getInt(
				EnvironmentParams.CHECKPOINTER_RETRY);
	}

	/**
	 * Called whenever the DaemonThread wakes up from a sleep.
	 */
	protected void onWakeup() throws DatabaseException {

		if (envImpl.isClosed()) {
			return;
		}

		checkpointer.doCheckpoint(CheckpointConfig.DEFAULT, false, // flushAll
				"daemon");
	}
	
	synchronized public void clearEnv() {
		envImpl = null;
	}
}
