package com.sleepycat.je.recovery;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.config.BooleanConfigParam;
import com.sleepycat.je.config.EnvironmentParams;
import com.sleepycat.je.config.IntConfigParam;
import com.sleepycat.je.dbi.DbConfigManager;
import com.sleepycat.je.dbi.EnvironmentImpl;

public privileged aspect DaemonInfrastructure {

	public static final IntConfigParam EnvironmentParams.CHECKPOINTER_RETRY = new IntConfigParam(
			"je.checkpointer.deadlockRetry", new Integer(0), // miyn
			new Integer(Integer.MAX_VALUE), // max
			new Integer(3), // default
			false, // mutable
			"# The number of times to retry a checkpoint if it runs into a deadlock.");

	private CheckpointerDaemon EnvironmentImpl.checkpointerDaemon;

	public CheckpointerDaemon EnvironmentImpl.getCheckpointerDaemon() {
		return checkpointerDaemon;
	}

	before(EnvironmentImpl envImpl) throws DatabaseException:
		execution(void EnvironmentImpl.createDaemons()) && this(envImpl) {

		/* Checkpointer */
		envImpl.checkpointerDaemon = new CheckpointerDaemon(
				envImpl.checkpointer, "Checkpointer");
	}

	before(EnvironmentImpl envImpl, DbConfigManager mgr)
			throws DatabaseException:
		execution(void EnvironmentImpl.runOrPauseDaemons(DbConfigManager)) && args(mgr) && this(envImpl) {

		/*
		 * Checkpointer. Run in both transactional and non-transactional
		 * environments to guarantee recovery time.
		 */
		if (!envImpl.isReadOnly)
			envImpl.checkpointerDaemon.runOrPause(mgr
					.getBoolean(EnvironmentParams.ENV_RUN_CHECKPOINTER));

	}

	public static final BooleanConfigParam EnvironmentParams.ENV_RUN_CHECKPOINTER = new BooleanConfigParam(
			"je.env.runCheckpointer", true, // default
			true, // mutable
			"# If true, starts up the checkpointer.\n"
					+ "# This parameter is true by default");

	void EnvironmentImpl.shutdownCheckpointerDaemon()
			throws InterruptedException {

		if (checkpointerDaemon != null) {
			checkpointerDaemon.shutdown();

			/*
			 * If daemon thread doesn't shutdown for any reason, at least clear
			 * the reference to the environment so it can be GC'd.
			 */
			checkpointerDaemon.clearEnv();
			checkpointerDaemon = null;
		}
		return;
	}

	after(EnvironmentImpl envImpl) throws InterruptedException:
		execution(void EnvironmentImpl.shutdownDaemons()) && this(envImpl) {

		envImpl.shutdownCheckpointerDaemon();
	}

	after(EnvironmentImpl envImpl) :
		execution(void EnvironmentImpl.requestShutdownDaemons()) && this(envImpl) 
	{
		if (envImpl.checkpointerDaemon != null) {
			envImpl.checkpointerDaemon.requestShutdown();
		}
	}
}
